/*****************************************************************************
 *  File: main.c
 *
 *  This file is the top level of the application selector.
 *
 ****************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "io.h"
#include "reconfig_utils.h"

// to help debug
#ifdef DEBUG_MODE
#include "errno.h"
#endif

// UCOS-II definitions
#include "includes.h"
#include "os_utils.h"

// Nichestack definitions
#include "ipport.h"
#include "libport.h"
#include "osport.h"
#include "tcpport.h"
#include "net.h"
#include "webserver.h"

#include "system.h"

// Task Stacks
OS_STK led_task_stk[LED_TASK_STACKSIZE];

// printf semaphore.
extern OS_EVENT *OSPrintfSem;

// Network init.
extern void NETInit();


void LedTask(void* pdata)
{
    char led_value = 0x5;

#ifdef DEBUG_MODE
    printf("DEBUG: LEDTask \n");
#endif

    IOWR(LED_PIO_BASE, 0, 0xffffffff);

    while (1) {
        // Flip the led
        led_value ^= 0xF;

        // Now display it.
        IOWR(LED_PIO_BASE, 0, led_value);

        // Wait 1/4 second
        OSTimeDlyHMSM(0, 0, 0, 250);
    }
}

/****************************************************************************
 *  Function: InitialTask
 *
 *  Purpose: InitialTask will initialize the NichStack TCP/IP stack and
 *           then initialize the rest of the tasks.
 *
 *  Returns: void
 ****************************************************************************/
void InitialTask(void* pdata)
{
#ifdef DEBUG_MODE
    printf("DEBUG: InitialTask0 \n");
#endif

    INT8U return_code = OS_NO_ERR;

    // Initialize the printf semaphore
    OSPrintfSem = OSSemCreate(1);

    /****************************************************************************
     *  Create the system tasks.
     ****************************************************************************/
    return_code = OSTaskCreateExt(LedTask,
            NULL,
            (void *) &led_task_stk[LED_TASK_STACKSIZE - 1],
            LED_PRIO,
            LED_PRIO,
            led_task_stk,
            LED_TASK_STACKSIZE,
            NULL,
            OS_TASK_OPT_STK_CHK + OS_TASK_OPT_STK_CLR);
    alt_uCOSIIErrorHandler(return_code, 0);

#ifdef DEBUG_MODE
    printf("DEBUG: InitialTask alt_uCOSIIErrorHandler \n");
#endif

    // Initialize the Network.
    NETInit();

    // This task deletes itself, since there's no reason to keep it around, once it's complete.
    return_code = OSTaskDel(OS_PRIO_SELF);
    alt_uCOSIIErrorHandler(return_code, 0);

    while (1); // Correct Program Flow should not reach here.
}

int main()
{

    INT8U return_code;

#ifdef DEBUG_MODE
    printf("DEBUG: RUNNING IN PRINTF DEBUG MODE! \n");
    printf("DEBUG: main \n");
#endif

    //unsigned int  *base = (unsigned int *)0x0;
    //IOWR_32DIRECT(base, 0, 0x00FF00FF);  // reset

    OS_STK InitialTaskStk[INITIAL_TASK_STACKSIZE];

    /* Clear the RTOS timer */
    OSTimeSet(0);

    return_code = OSTaskCreateExt(InitialTask,
            NULL,
            (void *) &InitialTaskStk[INITIAL_TASK_STACKSIZE - 1],
            INITIAL_TASK_PRIO,
            INITIAL_TASK_PRIO,
            InitialTaskStk,
            INITIAL_TASK_STACKSIZE,
            NULL,
            0);
    alt_uCOSIIErrorHandler(return_code, 0);

    /* As with all MicroC/OS-II designs, once the initial thread(s) and
     * associated RTOS resources are declared, we start the RTOS. That's it!
     */
    OSStart();

    while (1); /* Correct Program Flow never gets here. */

    return 0;
}

